"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Color_1 = require("./Color");
const COLOR_PALETTE_LENGTH = 15;
class ColorPalette {
    constructor(acnlBytes) {
        this.colors = new Array(COLOR_PALETTE_LENGTH + 1);
        for (let i = 0; i < COLOR_PALETTE_LENGTH; i++) {
            this.colors[i] = new Color_1.Color;
        }
        this.colors[15] = Color_1.Color.Transparent;
        if (acnlBytes) {
            this.acnlBytes = acnlBytes;
        }
    }
    get acnlBytes() {
        return new Uint8Array(this.colors.slice(0, COLOR_PALETTE_LENGTH).map(c => c.acnlByte));
    }
    set acnlBytes(value) {
        if (value.length !== COLOR_PALETTE_LENGTH) {
            throw new Error(`Invalid color palette length, expected ${COLOR_PALETTE_LENGTH} bytes, got ${value.length} bytes`);
        }
        for (let i = 0; i < COLOR_PALETTE_LENGTH; i++) {
            this.colors[i].acnlByte = value[i];
        }
    }
}
exports.ColorPalette = ColorPalette;
