"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DesignType_1 = require("./DesignType");
const ImageSize_1 = require("./ImageSize");
class DesignUsage {
    constructor(params) {
        this.id = params.id;
        this.description = params.description;
        this.type = params.type;
        this.imageSize = params.imageSize;
    }
    get width() {
        return 32;
    }
    get height() {
        return (this.imageSize === ImageSize_1.ImageSize.Large) ? 128 : 32;
    }
    get pixelCount() {
        return this.width * this.height;
    }
    get imageDataByteLength() {
        return this.pixelCount / 2;
    }
    get byteLength() {
        return (this.imageSize === ImageSize_1.ImageSize.Large) ? 2160 : 620;
    }
    get qrCodeCount() {
        return (this.imageSize === ImageSize_1.ImageSize.Large) ? 4 : 1;
    }
    static fromId(id) {
        return this.all.find(d => d.id === id);
    }
}
exports.DesignUsage = DesignUsage;
DesignUsage.LongSleeveDress = new DesignUsage({
    id: 0,
    description: 'Long-sleeve dress',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.ShortSleeveDress = new DesignUsage({
    id: 1,
    description: 'Short-sleeve dress',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.SleevelessDress = new DesignUsage({
    id: 2,
    description: 'Sleeveless dress',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.LongSleeveShirt = new DesignUsage({
    id: 3,
    description: 'Long-sleeve shirt',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.ShortSleeveShirt = new DesignUsage({
    id: 4,
    description: 'Short-sleeve shirt',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.SleevelessShirt = new DesignUsage({
    id: 5,
    description: 'Sleeveless shirt',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.HornedHat = new DesignUsage({
    id: 6,
    description: 'Horned hat',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Normal,
});
DesignUsage.KnitCap = new DesignUsage({
    id: 7,
    description: 'Knit cap',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Normal,
});
DesignUsage.Standee = new DesignUsage({
    id: 8,
    description: 'Standee',
    type: DesignType_1.DesignType.Pro,
    imageSize: ImageSize_1.ImageSize.Large,
});
DesignUsage.CustomDesign = new DesignUsage({
    id: 9,
    description: 'Custom design',
    type: DesignType_1.DesignType.Normal,
    imageSize: ImageSize_1.ImageSize.Normal,
});
DesignUsage.UnknownUsage010 = new DesignUsage({
    id: 10,
    description: 'Unknown Usage #10',
    type: DesignType_1.DesignType.Normal,
    imageSize: ImageSize_1.ImageSize.Normal,
});
DesignUsage.UnknownUsage011 = new DesignUsage({
    id: 11,
    description: 'Unknown Usage #11',
    type: DesignType_1.DesignType.Normal,
    imageSize: ImageSize_1.ImageSize.Normal,
});
DesignUsage.all = [
    DesignUsage.LongSleeveDress,
    DesignUsage.ShortSleeveDress,
    DesignUsage.SleevelessDress,
    DesignUsage.LongSleeveShirt,
    DesignUsage.ShortSleeveShirt,
    DesignUsage.SleevelessShirt,
    DesignUsage.HornedHat,
    DesignUsage.KnitCap,
    DesignUsage.Standee,
    DesignUsage.CustomDesign,
    DesignUsage.UnknownUsage010,
    DesignUsage.UnknownUsage011,
];
