"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const IndexedImageBase_1 = require("./IndexedImageBase");
const TRANSPARENT_COLOR_INDEX = 15;
class CompositeIndexedImage extends IndexedImageBase_1.IndexedImageBase {
    constructor() {
        super(...arguments);
        this.sources = [];
    }
    addSource(image, x, y) {
        this.validateCoordinates(x, y);
        this.sources.push({ image, x, y });
        return this;
    }
    get colorPalette() {
        // Just return the first source's color palette, since all sources should use the same palette
        return this.sources[0].image.colorPalette;
    }
    getSourceAt(x, y) {
        return this.sources.find(s => s.x <= x && s.x + s.image.width - 1 >= x &&
            s.y <= y && s.y + s.image.height - 1 >= y);
    }
    getColorIndexAt(x, y) {
        this.validateCoordinates(x, y);
        const source = this.getSourceAt(x, y);
        return (source) ? source.image.getColorIndexAt(x - source.x, y - source.y) : TRANSPARENT_COLOR_INDEX;
    }
    setColorIndexAt(x, y, value) {
        this.validateCoordinates(x, y);
        const source = this.getSourceAt(x, y);
        if (source) {
            source.image.setColorIndexAt(x - source.x, y - source.y, value);
        }
    }
}
exports.CompositeIndexedImage = CompositeIndexedImage;
