"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const canvas_1 = require("canvas");
const XbrzImageFilter_1 = require("./XbrzImageFilter");
const ByteUtils_1 = require("../support/ByteUtils");
// Compatibility layer for type checking in the browser
const CanvasRenderingContext2D = canvas_1.CanvasRenderingContext2D || window.CanvasRenderingContext2D;
class Image {
    constructor(width, height, data) {
        this.width = width;
        this.height = height;
        this.data = data;
    }
    static fromCanvas(canvas) {
        const imageData = canvas.getContext('2d').getImageData(0, 0, canvas.width, canvas.height);
        return new Image(imageData.width, imageData.height, ByteUtils_1.convertPixelsFrom8To32(imageData.data));
    }
    // Filters
    applyFilter(filter) {
        return filter.apply(this);
    }
    applyXbrzUpscaling(scale = 4) {
        return this.applyFilter(new XbrzImageFilter_1.XbrzImageFilter(scale));
    }
    toCanvas(value, x = 0, y = 0) {
        const context = this.resolveContext(value);
        context.putImageData(this.getImageData(), x, y);
        return context.canvas;
    }
    toDataURL(mimeType = 'image/png') {
        return this.toCanvas().toDataURL(mimeType);
    }
    toBlob(mimeType) {
        return new Promise((resolve, reject) => {
            this.toCanvas().toBlob((blob) => {
                if (blob)
                    resolve(blob);
                else
                    reject();
            }, mimeType);
        });
    }
    toBuffer(mimeType = 'image/png') {
        return new Promise((resolve, reject) => {
            this.toCanvas().toBuffer((err, result) => {
                if (err)
                    reject(err);
                else
                    resolve(result);
            }, mimeType);
        });
    }
    toFile(path, mimeType) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!mimeType) {
                mimeType = /\.jpe?g$/i.test(path) ? 'image/jpeg' : 'image/png';
            }
            const buffer = yield this.toBuffer(mimeType);
            return fs_1.promises.writeFile(path, buffer);
        });
    }
    resolveCanvas(canvas) {
        return (canvas) ? canvas : canvas_1.createCanvas(this.width, this.height);
    }
    resolveContext(value) {
        if (value instanceof CanvasRenderingContext2D) {
            return value;
        }
        return this.resolveCanvas(value).getContext('2d');
    }
    getImageData() {
        const data = ByteUtils_1.convertPixelsFrom32To8(this.data);
        return canvas_1.createImageData(data, this.width, this.height);
    }
}
exports.Image = Image;
