"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Image_1 = require("./Image");
const __1 = require("..");
class IndexedImageBase {
    constructor(width, height) {
        this.width = width;
        this.height = height;
    }
    dataSize() {
        return this.width * this.height;
    }
    getPixelIndex(x, y) {
        this.validateCoordinates(x, y);
        return x + (y * this.width);
    }
    get colorIndexes() {
        const result = new Uint8Array(this.dataSize());
        for (let x = 0; x < this.width; x++) {
            for (let y = 0; y < this.height; y++) {
                result[this.getPixelIndex(x, y)] = this.getColorIndexAt(x, y);
            }
        }
        return result;
    }
    set colorIndexes(value) {
        this.validateDataSize(value);
        for (let x = 0; x < this.width; x++) {
            for (let y = 0; y < this.height; y++) {
                this.setColorIndexAt(x, y, value[this.getPixelIndex(x, y)]);
            }
        }
    }
    getSegment(x, y, width, height) {
        return new __1.IndexedImageSegment(this, x, y, width, height);
    }
    toImage() {
        // Get the RGBA hex values for each color in the palette
        const palette = this.colorPalette.colors.map(c => c.hexNumber);
        palette.push(0x00000000); // The last palette index is reserved for transparency
        // Create an array of pixel data for the converted color data values
        const imageData = Uint32Array.from(this.colorIndexes, i => palette[i]);
        return new Image_1.Image(this.width, this.height, imageData);
    }
    validateDataSize(data) {
        if (data.length !== this.dataSize()) {
            throw new Error(`Invalid data size, expected ${this.dataSize()} bytes, got ${data.length} bytes`);
        }
    }
    validateCoordinates(x, y) {
        if (x < 0 || x >= this.width) {
            throw new Error(`x coordinate out of bounds, must be between 0 and ${this.width - 1}`);
        }
        if (y < 0 || y >= this.height) {
            throw new Error(`y coordinate out of bounds, must be between 0 and ${this.height - 1}`);
        }
    }
}
exports.IndexedImageBase = IndexedImageBase;
